@@ -9,6 +9,8 @@ cmake -DENABLE_SPARKLE_UPDATER=ON \
 -DCMAKE_OSX_DEPLOYMENT_TARGET=10.11 \
 -DQTDIR=/usr/local/Cellar/qt/5.10.1 \
 -DDepsPath=/tmp/obsdeps \
--DVLCPath=$PWD/../../vlc-master \
+-DVLCPath=$PWD/../../vlc-3.0.4 \
 -DBUILD_BROWSER=ON \
+-DBROWSER_DEPLOY=ON \
+-DBUILD_CAPTIONS=ON \
 -DCEF_ROOT_DIR=$PWD/../../cef_binary_${CEF_BUILD_VERSION}_macosx64 ..
