@@ -8,6 +8,12 @@
 
 using namespace std;
 
+extern bool EncoderAvailable(const char *encoder);
+
+volatile bool streaming_active = false;
+volatile bool recording_active = false;
+volatile bool replaybuf_active = false;
+
 static void OBSStreamStarting(void *data, calldata_t *params)
 {
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
@@ -39,6 +45,7 @@ static void OBSStartStreaming(void *data, calldata_t *params)
 {
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
 	output->streamingActive = true;
+	os_atomic_set_bool(&streaming_active, true);
 	QMetaObject::invokeMethod(output->main, "StreamingStart");
 
 	UNUSED_PARAMETER(params);
@@ -54,8 +61,10 @@ static void OBSStopStreaming(void *data, calldata_t *params)
 
 	output->streamingActive = false;
 	output->delayActive = false;
+	os_atomic_set_bool(&streaming_active, false);
 	QMetaObject::invokeMethod(output->main,
-			"StreamingStop", Q_ARG(int, code), Q_ARG(QString, arg_last_error));
+			"StreamingStop", Q_ARG(int, code),
+			Q_ARG(QString, arg_last_error));
 }
 
 static void OBSStartRecording(void *data, calldata_t *params)
@@ -63,6 +72,7 @@ static void OBSStartRecording(void *data, calldata_t *params)
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
 
 	output->recordingActive = true;
+	os_atomic_set_bool(&recording_active, true);
 	QMetaObject::invokeMethod(output->main, "RecordingStart");
 
 	UNUSED_PARAMETER(params);
@@ -72,10 +82,15 @@ static void OBSStopRecording(void *data, calldata_t *params)
 {
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
 	int code = (int)calldata_int(params, "code");
+	const char *last_error = calldata_string(params, "last_error");
+
+	QString arg_last_error = QString::fromUtf8(last_error);
 
 	output->recordingActive = false;
+	os_atomic_set_bool(&recording_active, false);
 	QMetaObject::invokeMethod(output->main,
-			"RecordingStop", Q_ARG(int, code));
+			"RecordingStop", Q_ARG(int, code),
+			Q_ARG(QString, arg_last_error));
 
 	UNUSED_PARAMETER(params);
 }
@@ -93,6 +108,7 @@ static void OBSStartReplayBuffer(void *data, calldata_t *params)
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
 
 	output->replayBufferActive = true;
+	os_atomic_set_bool(&replaybuf_active, true);
 	QMetaObject::invokeMethod(output->main, "ReplayBufferStart");
 
 	UNUSED_PARAMETER(params);
@@ -104,6 +120,7 @@ static void OBSStopReplayBuffer(void *data, calldata_t *params)
 	int code = (int)calldata_int(params, "code");
 
 	output->replayBufferActive = false;
+	os_atomic_set_bool(&replaybuf_active, false);
 	QMetaObject::invokeMethod(output->main,
 			"ReplayBufferStop", Q_ARG(int, code));
 
@@ -242,6 +259,8 @@ void SimpleOutput::LoadRecordingPreset_Lossless()
 	obs_data_set_string(settings, "video_encoder", "utvideo");
 	obs_data_set_string(settings, "audio_encoder", "pcm_s16le");
 
+	int aMixes = 1;
+	obs_output_set_mixers(fileOutput, aMixes);
 	obs_output_update(fileOutput, settings);
 	obs_data_release(settings);
 }
@@ -300,7 +319,10 @@ void SimpleOutput::LoadRecordingPreset()
 		} else if (strcmp(encoder, SIMPLE_ENCODER_AMD) == 0) {
 			LoadRecordingPreset_h264("amd_amf_h264");
 		} else if (strcmp(encoder, SIMPLE_ENCODER_NVENC) == 0) {
-			LoadRecordingPreset_h264("ffmpeg_nvenc");
+			const char *id = EncoderAvailable("jim_nvenc")
+				? "jim_nvenc"
+				: "ffmpeg_nvenc";
+			LoadRecordingPreset_h264(id);
 		}
 		usingRecordingPreset = true;
 
@@ -315,14 +337,22 @@ SimpleOutput::SimpleOutput(OBSBasic *main_) : BasicOutputHandler(main_)
 {
 	const char *encoder = config_get_string(main->Config(), "SimpleOutput",
 			"StreamEncoder");
-	if (strcmp(encoder, SIMPLE_ENCODER_QSV) == 0)
+
+	if (strcmp(encoder, SIMPLE_ENCODER_QSV) == 0) {
 		LoadStreamingPreset_h264("obs_qsv11");
-	else if (strcmp(encoder, SIMPLE_ENCODER_AMD) == 0)
+
+	} else if (strcmp(encoder, SIMPLE_ENCODER_AMD) == 0) {
 		LoadStreamingPreset_h264("amd_amf_h264");
-	else if (strcmp(encoder, SIMPLE_ENCODER_NVENC) == 0)
-		LoadStreamingPreset_h264("ffmpeg_nvenc");
-	else
+
+	} else if (strcmp(encoder, SIMPLE_ENCODER_NVENC) == 0) {
+		const char *id = EncoderAvailable("jim_nvenc")
+			? "jim_nvenc"
+			: "ffmpeg_nvenc";
+		LoadStreamingPreset_h264(id);
+
+	} else {
 		LoadStreamingPreset_h264("obs_x264");
+	}
 
 	if (!CreateAACEncoder(aacStreaming, aacStreamEncID, GetAudioBitrate(),
 				"simple_aac", 0))
@@ -649,6 +679,10 @@ bool SimpleOutput::StartStreaming(obs_service_t *service)
 	if (!Active())
 		SetupOutputs();
 
+	Auth *auth = main->GetAuth();
+	if (auth)
+		auth->OnStreamConfig();
+
 	/* --------------------- */
 
 	const char *type = obs_service_get_output_type(service);
@@ -685,28 +719,39 @@ bool SimpleOutput::StartStreaming(obs_service_t *service)
 				obs_output_get_signal_handler(streamOutput),
 				"stop", OBSStopStreaming, this);
 
-		const char *codec =
-			obs_output_get_supported_audio_codecs(streamOutput);
-		if (!codec) {
-			return false;
-		}
+		bool isEncoded = obs_output_get_flags(streamOutput)
+			& OBS_OUTPUT_ENCODED;
 
-		if (strcmp(codec, "aac") != 0) {
-			const char *id = FindAudioEncoderFromCodec(codec);
-			int audioBitrate = GetAudioBitrate();
-			obs_data_t *settings = obs_data_create();
-			obs_data_set_int(settings, "bitrate", audioBitrate);
-
-			aacStreaming = obs_audio_encoder_create(id,
-					"alt_audio_enc", nullptr, 0, nullptr);
-			obs_encoder_release(aacStreaming);
-			if (!aacStreaming)
+		if (isEncoded) {
+			const char *codec =
+				obs_output_get_supported_audio_codecs(
+						streamOutput);
+			if (!codec) {
+				blog(LOG_WARNING, "Failed to load audio codec");
 				return false;
+			}
 
-			obs_encoder_update(aacStreaming, settings);
-			obs_encoder_set_audio(aacStreaming, obs_get_audio());
+			if (strcmp(codec, "aac") != 0) {
+				const char *id = FindAudioEncoderFromCodec(
+						codec);
+				int audioBitrate = GetAudioBitrate();
+				obs_data_t *settings = obs_data_create();
+				obs_data_set_int(settings, "bitrate",
+						audioBitrate);
 
-			obs_data_release(settings);
+				aacStreaming = obs_audio_encoder_create(id,
+						"alt_audio_enc", nullptr, 0,
+						nullptr);
+				obs_encoder_release(aacStreaming);
+				if (!aacStreaming)
+					return false;
+
+				obs_encoder_update(aacStreaming, settings);
+				obs_encoder_set_audio(aacStreaming,
+						obs_get_audio());
+
+				obs_data_release(settings);
+			}
 		}
 
 		outputType = type;
@@ -760,12 +805,15 @@ bool SimpleOutput::StartStreaming(obs_service_t *service)
 	}
 
 	const char *error = obs_output_get_last_error(streamOutput);
-	bool has_last_error = error && *error;
+	bool hasLastError = error && *error;
+	if (hasLastError)
+		lastError = error;
+	else
+		lastError = string();
 
-	blog(LOG_WARNING, "Stream output type '%s' failed to start!%s%s",
-			type,
-			has_last_error ? "  Last Error: " : "",
-			has_last_error ? error : "");
+	blog(LOG_WARNING, "Stream output type '%s' failed to start!%s%s", type,
+			hasLastError ? "  Last Error: " : "",
+			hasLastError ? error : "");
 	return false;
 }
 
@@ -848,7 +896,7 @@ bool SimpleOutput::ConfigureRecording(bool updateReplayBuffer)
 
 	if (!dir) {
 		if (main->isVisible())
-			OBSMessageBox::information(main,
+			OBSMessageBox::warning(main,
 					QTStr("Output.BadPath.Title"),
 					QTStr("Output.BadPath.Text"));
 		else
@@ -1207,9 +1255,14 @@ inline void AdvancedOutput::SetupStreaming()
 			"Rescale");
 	const char *rescaleRes = config_get_string(main->Config(), "AdvOut",
 			"RescaleRes");
+	uint32_t caps = obs_encoder_get_caps(h264Streaming);
 	unsigned int cx = 0;
 	unsigned int cy = 0;
 
+	if ((caps & OBS_ENCODER_CAP_PASS_TEXTURE) != 0) {
+		rescale = false;
+	}
+
 	if (rescale && rescaleRes && *rescaleRes) {
 		if (sscanf(rescaleRes, "%ux%u", &cx, &cy) != 2) {
 			cx = 0;
@@ -1243,6 +1296,11 @@ inline void AdvancedOutput::SetupRecording()
 			obs_output_set_video_encoder(replayBuffer,
 					h264Streaming);
 	} else {
+		uint32_t caps = obs_encoder_get_caps(h264Recording);
+		if ((caps & OBS_ENCODER_CAP_PASS_TEXTURE) != 0) {
+			rescale = false;
+		}
+
 		if (rescale && rescaleRes && *rescaleRes) {
 			if (sscanf(rescaleRes, "%ux%u", &cx, &cy) != 2) {
 				cx = 0;
@@ -1302,8 +1360,8 @@ inline void AdvancedOutput::SetupFFmpeg()
 			"FFVCustom");
 	int aBitrate = config_get_int(main->Config(), "AdvOut",
 			"FFABitrate");
-	int aTrack = config_get_int(main->Config(), "AdvOut",
-			"FFAudioTrack");
+	int aMixes = config_get_int(main->Config(), "AdvOut",
+			"FFAudioMixes");
 	const char *aEncoder = config_get_string(main->Config(), "AdvOut",
 			"FFAEncoder");
 	int aEncoderId = config_get_int(main->Config(), "AdvOut",
@@ -1337,7 +1395,7 @@ inline void AdvancedOutput::SetupFFmpeg()
 		}
 	}
 
-	obs_output_set_mixer(fileOutput, aTrack - 1);
+	obs_output_set_mixers(fileOutput, aMixes);
 	obs_output_set_media(fileOutput, obs_get_video(), obs_get_audio());
 	obs_output_update(fileOutput, settings);
 
@@ -1424,6 +1482,10 @@ bool AdvancedOutput::StartStreaming(obs_service_t *service)
 	if (!Active())
 		SetupOutputs();
 
+	Auth *auth = main->GetAuth();
+	if (auth)
+		auth->OnStreamConfig();
+
 	/* --------------------- */
 
 	int trackIndex = config_get_int(main->Config(), "AdvOut",
@@ -1463,30 +1525,42 @@ bool AdvancedOutput::StartStreaming(obs_service_t *service)
 				obs_output_get_signal_handler(streamOutput),
 				"stop", OBSStopStreaming, this);
 
-		const char *codec =
-			obs_output_get_supported_audio_codecs(streamOutput);
-		if (!codec) {
-			return false;
-		}
+		bool isEncoded = obs_output_get_flags(streamOutput)
+			& OBS_OUTPUT_ENCODED;
 
-		if (strcmp(codec, "aac") == 0) {
-			streamAudioEnc = aacTrack[trackIndex - 1];
-		} else {
-			const char *id = FindAudioEncoderFromCodec(codec);
-			int audioBitrate = GetAudioBitrate(trackIndex - 1);
-			obs_data_t *settings = obs_data_create();
-			obs_data_set_int(settings, "bitrate", audioBitrate);
-
-			streamAudioEnc = obs_audio_encoder_create(id,
-					"alt_audio_enc", nullptr,
-					trackIndex - 1, nullptr);
-			if (!streamAudioEnc)
+		if (isEncoded) {
+			const char *codec =
+				obs_output_get_supported_audio_codecs(
+						streamOutput);
+			if (!codec) {
+				blog(LOG_WARNING, "Failed to load audio codec");
 				return false;
+			}
 
-			obs_encoder_update(streamAudioEnc, settings);
-			obs_encoder_set_audio(streamAudioEnc, obs_get_audio());
+			if (strcmp(codec, "aac") == 0) {
+				streamAudioEnc = aacTrack[trackIndex - 1];
+			} else {
+				obs_data_t *settings = obs_data_create();
+				const char *id =
+					FindAudioEncoderFromCodec(codec);
+				int audioBitrate =
+					GetAudioBitrate(trackIndex - 1);
+				
+				obs_data_set_int(settings, "bitrate",
+						audioBitrate);
+				streamAudioEnc = obs_audio_encoder_create(id,
+						"alt_audio_enc", nullptr,
+						trackIndex - 1, nullptr);
 
-			obs_data_release(settings);
+				if (!streamAudioEnc)
+					return false;
+
+				obs_encoder_update(streamAudioEnc, settings);
+				obs_encoder_set_audio(streamAudioEnc,
+						obs_get_audio());
+
+				obs_data_release(settings);
+			}
 		}
 
 		outputType = type;
@@ -1538,12 +1612,15 @@ bool AdvancedOutput::StartStreaming(obs_service_t *service)
 	}
 
 	const char *error = obs_output_get_last_error(streamOutput);
-	bool has_last_error = error && *error;
+	bool hasLastError = error && *error;
+	if (hasLastError)
+		lastError = error;
+	else
+		lastError = string();
 
-	blog(LOG_WARNING, "Stream output type '%s' failed to start!%s%s",
-			type,
-			has_last_error ? "  Last Error: " : "",
-			has_last_error ? error : "");
+	blog(LOG_WARNING, "Stream output type '%s' failed to start!%s%s", type,
+			hasLastError ? "  Last Error: " : "",
+			hasLastError ? error : "");
 	return false;
 }
 
@@ -1586,7 +1663,7 @@ bool AdvancedOutput::StartRecording()
 
 		if (!dir) {
 			if (main->isVisible())
-				OBSMessageBox::information(main,
+				OBSMessageBox::warning(main,
 						QTStr("Output.BadPath.Title"),
 						QTStr("Output.BadPath.Text"));
 			else
@@ -1686,7 +1763,7 @@ bool AdvancedOutput::StartReplayBuffer()
 
 		if (!dir) {
 			if (main->isVisible())
-				OBSMessageBox::information(main,
+				OBSMessageBox::warning(main,
 						QTStr("Output.BadPath.Title"),
 						QTStr("Output.BadPath.Text"));
 			else
