@@ -395,6 +395,8 @@ static void update_item_transform(struct obs_scene_item *item, bool update_tex)
 		scale.y = (float)height * item->scale.y;
 	}
 
+	item->box_scale = scale;
+
 	add_alignment(&base_origin, item->align, (int)scale.x, (int)scale.y);
 
 	matrix4_identity(&item->box_transform);
@@ -462,6 +464,8 @@ static inline bool item_texture_enabled(const struct obs_scene_item *item)
 
 static void render_item_texture(struct obs_scene_item *item)
 {
+	GS_DEBUG_MARKER_BEGIN(GS_DEBUG_COLOR_ITEM_TEXTURE, "render_item_texture");
+
 	gs_texture_t *tex = gs_texrender_get_texture(item->item_render);
 	gs_effect_t *effect = obs->video.default_effect;
 	enum obs_scale_type type = item->scale_filter;
@@ -486,6 +490,8 @@ static void render_item_texture(struct obs_scene_item *item)
 				effect = obs->video.bicubic_effect;
 			} else if (type == OBS_SCALE_LANCZOS) {
 				effect = obs->video.lanczos_effect;
+			} else if (type == OBS_SCALE_AREA) {
+				effect = obs->video.area_effect;
 			}
 
 			scale_param = gs_effect_get_param_by_name(effect,
@@ -501,18 +507,29 @@ static void render_item_texture(struct obs_scene_item *item)
 		}
 	}
 
+	gs_blend_state_push();
+	gs_blend_function(GS_BLEND_ONE, GS_BLEND_INVSRCALPHA);
+
 	while (gs_effect_loop(effect, "Draw"))
 		obs_source_draw(tex, 0, 0, 0, 0, 0);
+
+	gs_blend_state_pop();
+
+	GS_DEBUG_MARKER_END();
 }
 
 static inline void render_item(struct obs_scene_item *item)
 {
+	GS_DEBUG_MARKER_BEGIN_FORMAT(GS_DEBUG_COLOR_ITEM, "Item: %s",
+			obs_source_get_name(item->source));
+
 	if (item->item_render) {
 		uint32_t width  = obs_source_get_width(item->source);
 		uint32_t height = obs_source_get_height(item->source);
 
-		if (!width || !height)
-			return;
+		if (!width || !height) {
+			goto cleanup;
+		}
 
 		uint32_t cx = calc_cx(item, width);
 		uint32_t cy = calc_cy(item, height);
@@ -533,10 +550,8 @@ static inline void render_item(struct obs_scene_item *item)
 					-(float)item->crop.top,
 					0.0f);
 
-			gs_blend_state_push();
-			gs_blend_function(GS_BLEND_ONE, GS_BLEND_ZERO);
 			obs_source_video_render(item->source);
-			gs_blend_state_pop();
+
 			gs_texrender_end(item->item_render);
 		}
 	}
@@ -549,6 +564,9 @@ static inline void render_item(struct obs_scene_item *item)
 		obs_source_video_render(item->source);
 	}
 	gs_matrix_pop();
+
+cleanup:
+	GS_DEBUG_MARKER_END();
 }
 
 static void scene_video_tick(void *data, float seconds)
@@ -748,6 +766,8 @@ static void scene_load_item(struct obs_scene *scene, obs_data_t *item_data)
 			item->scale_filter = OBS_SCALE_BICUBIC;
 		else if (astrcmpi(scale_filter_str, "lanczos") == 0)
 			item->scale_filter = OBS_SCALE_LANCZOS;
+		else if (astrcmpi(scale_filter_str, "area") == 0)
+			item->scale_filter = OBS_SCALE_AREA;
 	}
 
 	if (item->item_render && !item_texture_enabled(item)) {
@@ -857,6 +877,8 @@ static void scene_save_item(obs_data_array_t *array,
 		scale_filter = "bicubic";
 	else if (item->scale_filter == OBS_SCALE_LANCZOS)
 		scale_filter = "lanczos";
+	else if (item->scale_filter == OBS_SCALE_AREA)
+		scale_filter = "area";
 	else
 		scale_filter = "disable";
 
@@ -1243,6 +1265,7 @@ static inline void duplicate_item_data(struct obs_scene_item *dst,
 	dst->output_scale = src->output_scale;
 	dst->scale_filter = src->scale_filter;
 	dst->box_transform = src->box_transform;
+	dst->box_scale = src->box_scale;
 	dst->draw_transform = src->draw_transform;
 	dst->bounds_type = src->bounds_type;
 	dst->bounds_align = src->bounds_align;
@@ -2030,6 +2053,13 @@ void obs_sceneitem_get_box_transform(const obs_sceneitem_t *item,
 		matrix4_copy(transform, &item->box_transform);
 }
 
+void obs_sceneitem_get_box_scale(const obs_sceneitem_t *item,
+		struct vec2 *scale)
+{
+	if (item)
+		*scale = item->box_scale;
+}
+
 bool obs_sceneitem_visible(const obs_sceneitem_t *item)
 {
 	return item ? item->user_visible : false;
