@@ -58,8 +58,13 @@ static uint64_t tick_sources(uint64_t cur_time, uint64_t last_time)
 
 	source = data->first_source;
 	while (source) {
-		obs_source_video_tick(source, seconds);
+		struct obs_source *cur_source = obs_source_get_ref(source);
 		source = (struct obs_source*)source->context.next;
+
+		if (cur_source) {
+			obs_source_video_tick(cur_source, seconds);
+			obs_source_release(cur_source);
+		}
 	}
 
 	pthread_mutex_unlock(&data->sources_mutex);
@@ -115,9 +120,11 @@ static inline void render_main_texture(struct obs_core_video *video,
 		int cur_texture)
 {
 	profile_start(render_main_texture_name);
+	GS_DEBUG_MARKER_BEGIN(GS_DEBUG_COLOR_MAIN_TEXTURE,
+			render_main_texture_name);
 
 	struct vec4 clear_color;
-	vec4_set(&clear_color, 0.0f, 0.0f, 0.0f, 1.0f);
+	vec4_set(&clear_color, 0.0f, 0.0f, 0.0f, 0.0f);
 
 	gs_set_render_target(video->render_textures[cur_texture], NULL);
 	gs_clear(GS_CLEAR_COLOR, &clear_color, 1.0f, 0);
@@ -140,6 +147,7 @@ static inline void render_main_texture(struct obs_core_video *video,
 
 	video->textures_rendered[cur_texture] = true;
 
+	GS_DEBUG_MARKER_END();
 	profile_end(render_main_texture_name);
 }
 
@@ -207,7 +215,14 @@ static inline void render_output_texture(struct obs_core_video *video,
 		1.0f / (float)video->base_height);
 
 	gs_effect_t    *effect  = get_scale_effect(video, width, height);
-	gs_technique_t *tech    = gs_effect_get_technique(effect, "DrawMatrix");
+	gs_technique_t *tech;
+
+	if (video->ovi.output_format == VIDEO_FORMAT_RGBA) {
+		tech = gs_effect_get_technique(effect, "DrawAlphaDivide");
+	} else {
+		tech = gs_effect_get_technique(effect, "DrawMatrix");
+	}
+
 	gs_eparam_t    *image   = gs_effect_get_param_by_name(effect, "image");
 	gs_eparam_t    *matrix  = gs_effect_get_param_by_name(effect,
 			"color_matrix");
@@ -303,6 +318,57 @@ end:
 	profile_end(render_convert_texture_name);
 }
 
+static void render_nv12(struct obs_core_video *video, gs_texture_t *target,
+		int cur_texture, int prev_texture, const char *tech_name,
+		uint32_t width, uint32_t height)
+{
+	gs_texture_t *texture = video->output_textures[prev_texture];
+
+	gs_effect_t    *effect  = video->conversion_effect;
+	gs_eparam_t    *image   = gs_effect_get_param_by_name(effect, "image");
+	gs_technique_t *tech    = gs_effect_get_technique(effect, tech_name);
+	size_t         passes, i;
+
+	gs_effect_set_texture(image, texture);
+
+	gs_set_render_target(target, NULL);
+	set_render_size(width, height);
+
+	gs_enable_blending(false);
+	passes = gs_technique_begin(tech);
+	for (i = 0; i < passes; i++) {
+		gs_technique_begin_pass(tech, i);
+		gs_draw_sprite(texture, 0, width, height);
+		gs_technique_end_pass(tech);
+	}
+	gs_technique_end(tech);
+	gs_enable_blending(true);
+
+	UNUSED_PARAMETER(cur_texture);
+}
+
+static const char *render_convert_nv12_name = "render_convert_texture_nv12";
+static void render_convert_texture_nv12(struct obs_core_video *video,
+		int cur_texture, int prev_texture)
+{
+	profile_start(render_convert_nv12_name);
+
+	if (!video->textures_output[prev_texture])
+		goto end;
+
+	render_nv12(video, video->convert_textures[cur_texture],
+			cur_texture, prev_texture, "NV12_Y",
+			video->output_width, video->output_height);
+	render_nv12(video, video->convert_uv_textures[cur_texture],
+			cur_texture, prev_texture, "NV12_UV",
+			video->output_width / 2, video->output_height / 2);
+
+	video->textures_converted[cur_texture] = true;
+
+end:
+	profile_end(render_convert_nv12_name);
+}
+
 static const char *stage_output_texture_name = "stage_output_texture";
 static inline void stage_output_texture(struct obs_core_video *video,
 		int cur_texture, int prev_texture)
@@ -334,7 +400,100 @@ end:
 	profile_end(stage_output_texture_name);
 }
 
-static inline void render_video(struct obs_core_video *video, bool raw_active,
+#ifdef _WIN32
+static inline bool queue_frame(struct obs_core_video *video, bool raw_active,
+		struct obs_vframe_info *vframe_info, int prev_texture)
+{
+	bool duplicate = !video->gpu_encoder_avail_queue.size ||
+		(video->gpu_encoder_queue.size && vframe_info->count > 1);
+
+	if (duplicate) {
+		struct obs_tex_frame *tf = circlebuf_data(
+				&video->gpu_encoder_queue,
+				video->gpu_encoder_queue.size - sizeof(*tf));
+
+		/* texture-based encoding is stopping */
+		if (!tf) {
+			return false;
+		}
+
+		tf->count++;
+		os_sem_post(video->gpu_encode_semaphore);
+		goto finish;
+	}
+
+	struct obs_tex_frame tf;
+	circlebuf_pop_front(&video->gpu_encoder_avail_queue, &tf, sizeof(tf));
+
+	if (tf.released) {
+		gs_texture_acquire_sync(tf.tex, tf.lock_key, GS_WAIT_INFINITE);
+		tf.released = false;
+	}
+
+	/* the vframe_info->count > 1 case causing a copy can only happen if by
+	 * some chance the very first frame has to be duplicated for whatever
+	 * reason.  otherwise, it goes to the 'duplicate' case above, which
+	 * will ensure better performance. */
+	if (raw_active || vframe_info->count > 1) {
+		gs_copy_texture(tf.tex, video->convert_textures[prev_texture]);
+	} else {
+		gs_texture_t *tex = video->convert_textures[prev_texture];
+		gs_texture_t *tex_uv = video->convert_uv_textures[prev_texture];
+
+		video->convert_textures[prev_texture] = tf.tex;
+		video->convert_uv_textures[prev_texture] = tf.tex_uv;
+
+		tf.tex = tex;
+		tf.tex_uv = tex_uv;
+	}
+
+	tf.count = 1;
+	tf.timestamp = vframe_info->timestamp;
+	tf.released = true;
+	tf.handle = gs_texture_get_shared_handle(tf.tex);
+	gs_texture_release_sync(tf.tex, ++tf.lock_key);
+	circlebuf_push_back(&video->gpu_encoder_queue, &tf, sizeof(tf));
+
+	os_sem_post(video->gpu_encode_semaphore);
+
+finish:
+	return --vframe_info->count;
+}
+
+extern void full_stop(struct obs_encoder *encoder);
+
+static inline void encode_gpu(struct obs_core_video *video, bool raw_active,
+		struct obs_vframe_info *vframe_info, int prev_texture)
+{
+	while (queue_frame(video, raw_active, vframe_info, prev_texture));
+}
+
+static const char *output_gpu_encoders_name = "output_gpu_encoders";
+static void output_gpu_encoders(struct obs_core_video *video, bool raw_active,
+		int prev_texture)
+{
+	profile_start(output_gpu_encoders_name);
+
+	if (!video->textures_converted[prev_texture])
+		goto end;
+	if (!video->vframe_info_buffer_gpu.size)
+		goto end;
+
+	struct obs_vframe_info vframe_info;
+	circlebuf_pop_front(&video->vframe_info_buffer_gpu, &vframe_info,
+			sizeof(vframe_info));
+
+	pthread_mutex_lock(&video->gpu_encoder_mutex);
+	encode_gpu(video, raw_active, &vframe_info, prev_texture);
+	pthread_mutex_unlock(&video->gpu_encoder_mutex);
+
+end:
+	profile_end(output_gpu_encoders_name);
+}
+#endif
+
+static inline void render_video(struct obs_core_video *video,
+		bool raw_active, const bool gpu_active,
 		int cur_texture, int prev_texture)
 {
 	gs_begin_scene();
@@ -344,12 +503,34 @@ static inline void render_video(struct obs_core_video *video, bool raw_active,
 
 	render_main_texture(video, cur_texture);
 
-	if (raw_active) {
+	if (raw_active || gpu_active) {
 		render_output_texture(video, cur_texture, prev_texture);
-		if (video->gpu_conversion)
-			render_convert_texture(video, cur_texture, prev_texture);
 
-		stage_output_texture(video, cur_texture, prev_texture);
+#ifdef _WIN32
+		if (gpu_active) {
+			gs_flush();
+		}
+#endif
+	}
+
+	if (raw_active || gpu_active) {
+		if (video->gpu_conversion) {
+			if (video->using_nv12_tex)
+				render_convert_texture_nv12(video,
+						cur_texture, prev_texture);
+			else
+				render_convert_texture(video,
+						cur_texture, prev_texture);
+		}
+
+#ifdef _WIN32
+		if (gpu_active) {
+			gs_flush();
+			output_gpu_encoders(video, raw_active, prev_texture);
+		}
+#endif
+		if (raw_active)
+			stage_output_texture(video, cur_texture, prev_texture);
 	}
 
 	gs_set_render_target(NULL, NULL);
@@ -448,6 +629,25 @@ static void set_gpu_converted_data(struct obs_core_video *video,
 
 		video_frame_copy(output, &frame, info->format, info->height);
 
+	} else if (video->using_nv12_tex) {
+		size_t width = info->width;
+		size_t height = info->height;
+		size_t height_d2 = height / 2;
+		uint8_t *out_y = output->data[0];
+		uint8_t *out_uv = output->data[1];
+		uint8_t *in = input->data[0];
+
+		for (size_t y = 0; y < height; y++) {
+			memcpy(out_y, in, width);
+			out_y += output->linesize[0];
+			in += input->linesize[0];
+		}
+		for (size_t y = 0; y < height_d2; y++) {
+			memcpy(out_uv, in, width);
+			out_uv += output->linesize[0];
+			in += input->linesize[0];
+		}
+
 	} else {
 		fix_gpu_converted_alignment(video, output, input);
 	}
@@ -525,7 +725,8 @@ static inline void output_video_data(struct obs_core_video *video,
 	}
 }
 
-static inline void video_sleep(struct obs_core_video *video, bool active,
+static inline void video_sleep(struct obs_core_video *video,
+		bool raw_active, const bool gpu_active,
 		uint64_t *p_time, uint64_t interval_ns)
 {
 	struct obs_vframe_info vframe_info;
@@ -546,9 +747,13 @@ static inline void video_sleep(struct obs_core_video *video, bool active,
 
 	vframe_info.timestamp = cur_time;
 	vframe_info.count = count;
-	if (active)
+
+	if (raw_active)
 		circlebuf_push_back(&video->vframe_info_buffer, &vframe_info,
 				sizeof(vframe_info));
+	if (gpu_active)
+		circlebuf_push_back(&video->vframe_info_buffer_gpu,
+				&vframe_info, sizeof(vframe_info));
 }
 
 static const char *output_frame_gs_context_name = "gs_context(video->graphics)";
@@ -556,13 +761,13 @@ static const char *output_frame_render_video_name = "render_video";
 static const char *output_frame_download_frame_name = "download_frame";
 static const char *output_frame_gs_flush_name = "gs_flush";
 static const char *output_frame_output_video_data_name = "output_video_data";
-static inline void output_frame(bool raw_active)
+static inline void output_frame(bool raw_active, const bool gpu_active)
 {
 	struct obs_core_video *video = &obs->video;
 	int cur_texture  = video->cur_texture;
 	int prev_texture = cur_texture == 0 ? NUM_TEXTURES-1 : cur_texture-1;
 	struct video_data frame;
-	bool frame_ready;
+	bool frame_ready = 0;
 
 	memset(&frame, 0, sizeof(struct video_data));
 
@@ -570,7 +775,10 @@ static inline void output_frame(bool raw_active)
 	gs_enter_context(video->graphics);
 
 	profile_start(output_frame_render_video_name);
-	render_video(video, raw_active, cur_texture, prev_texture);
+	GS_DEBUG_MARKER_BEGIN(GS_DEBUG_COLOR_RENDER_VIDEO,
+			output_frame_render_video_name);
+	render_video(video, raw_active, gpu_active, cur_texture, prev_texture);
+	GS_DEBUG_MARKER_END();
 	profile_end(output_frame_render_video_name);
 
 	if (raw_active) {
@@ -603,17 +811,31 @@ static inline void output_frame(bool raw_active)
 
 #define NBSP "\xC2\xA0"
 
-static void clear_frame_data(void)
+static void clear_base_frame_data(void)
 {
 	struct obs_core_video *video = &obs->video;
 	memset(video->textures_rendered, 0, sizeof(video->textures_rendered));
 	memset(video->textures_output, 0, sizeof(video->textures_output));
-	memset(video->textures_copied, 0, sizeof(video->textures_copied));
 	memset(video->textures_converted, 0, sizeof(video->textures_converted));
 	circlebuf_free(&video->vframe_info_buffer);
 	video->cur_texture = 0;
 }
 
+static void clear_raw_frame_data(void)
+{
+	struct obs_core_video *video = &obs->video;
+	memset(video->textures_copied, 0, sizeof(video->textures_copied));
+	circlebuf_free(&video->vframe_info_buffer);
+}
+
+#ifdef _WIN32
+static void clear_gpu_frame_data(void)
+{
+	struct obs_core_video *video = &obs->video;
+	circlebuf_free(&video->vframe_info_buffer_gpu);
+}
+#endif
+
 static const char *tick_sources_name = "tick_sources";
 static const char *render_displays_name = "render_displays";
 static const char *output_frame_name = "output_frame";
@@ -624,7 +846,11 @@ void *obs_graphics_thread(void *param)
 	uint64_t frame_time_total_ns = 0;
 	uint64_t fps_total_ns = 0;
 	uint32_t fps_total_frames = 0;
+#ifdef _WIN32
+	bool gpu_was_active = false;
+#endif
 	bool raw_was_active = false;
+	bool was_active = false;
 
 	obs->video.video_time = os_gettime_ns();
 
@@ -641,10 +867,26 @@ void *obs_graphics_thread(void *param)
 		uint64_t frame_start = os_gettime_ns();
 		uint64_t frame_time_ns;
 		bool raw_active = obs->video.raw_active > 0;
+#ifdef _WIN32
+		const bool gpu_active = obs->video.gpu_encoder_active > 0;
+		const bool active = raw_active || gpu_active;
+#else
+		const bool gpu_active = 0;
+		const bool active = raw_active;
+#endif
 
+		if (!was_active && active)
+			clear_base_frame_data();
 		if (!raw_was_active && raw_active)
-			clear_frame_data();
+			clear_raw_frame_data();
+#ifdef _WIN32
+		if (!gpu_was_active && gpu_active)
+			clear_gpu_frame_data();
+
+		gpu_was_active = gpu_active;
+#endif
 		raw_was_active = raw_active;
+		was_active = active;
 
 		profile_start(video_thread_name);
 
@@ -653,7 +895,7 @@ void *obs_graphics_thread(void *param)
 		profile_end(tick_sources_name);
 
 		profile_start(output_frame_name);
-		output_frame(raw_active);
+		output_frame(raw_active, gpu_active);
 		profile_end(output_frame_name);
 
 		profile_start(render_displays_name);
@@ -666,8 +908,8 @@ void *obs_graphics_thread(void *param)
 
 		profile_reenable_thread();
 
-		video_sleep(&obs->video, raw_active, &obs->video.video_time,
-				interval);
+		video_sleep(&obs->video, raw_active, gpu_active,
+				&obs->video.video_time, interval);
 
 		frame_time_total_ns += frame_time_ns;
 		fps_total_ns += (obs->video.video_time - last_time);
