@@ -111,6 +111,8 @@ EXPORT char *os_get_config_path_ptr(const char *name);
 EXPORT int os_get_program_data_path(char *dst, size_t size, const char *name);
 EXPORT char *os_get_program_data_path_ptr(const char *name);
 
+EXPORT char *os_get_executable_path_ptr(const char *name);
+
 EXPORT bool os_file_exists(const char *path);
 
 EXPORT size_t os_get_abs_path(const char *path, char *abspath, size_t size);
@@ -181,6 +183,18 @@ EXPORT void os_breakpoint(void);
 EXPORT int os_get_physical_cores(void);
 EXPORT int os_get_logical_cores(void);
 
+EXPORT uint64_t os_get_sys_free_size(void);
+
+struct os_proc_memory_usage {
+	uint64_t resident_size;
+	uint64_t virtual_size;
+};
+typedef struct os_proc_memory_usage os_proc_memory_usage_t;
+
+EXPORT bool os_get_proc_memory_usage(os_proc_memory_usage_t *usage);
+EXPORT uint64_t os_get_proc_resident_size(void);
+EXPORT uint64_t os_get_proc_virtual_size(void);
+
 #ifdef _MSC_VER
 #define strtoll _strtoi64
 #if _MSC_VER < 1900
