@@ -155,6 +155,10 @@ static obs_properties_t *ffmpeg_source_getproperties(void *data)
 	obs_properties_add_bool(props, "restart_on_activate",
 			obs_module_text("RestartWhenActivated"));
 
+	obs_properties_add_int_slider(props, "buffering_mb",
+			obs_module_text("BufferingMB"),
+			1, 16, 1);
+
 	obs_properties_add_text(props, "input",
 			obs_module_text("Input"), OBS_TEXT_DEFAULT);
 
